﻿using System;
using System.Text;
using System.Xml;

namespace DefaultModelPlugin
{
    public class DefaultModelPlugin:Extension.Model.ModelFilePlugin{
        /// <summary>
        /// プラグイン名
        /// </summary>
        public override string PluginName{
            get {
                return "DefaultModelPlugin";
            }
        }

        /// <summary>
        /// 出力メニュータイトル
        /// </summary>
        public override string SaveMenuTitle{
            get{
                return "XML形式（3DMapEditor標準）"; 
            }
        }

        /// <summary>
        /// ボックスモデルデータの保存
        /// </summary>
        /// <param name="path">ディレクトリパス</param>
        /// <param name="infomation">ボックスモデル情報</param>
        public override void SaveModelFile(string path, Extension.Model.ModelInformation information){
            
            /*
             * informationからは以下の情報が取得できます。
             * ModelNo   : モデル番号 
             * ModelName : モデル名
             * Vertexs   : 頂点データ二次元配列
             *      Position : x,y,zベクトル位置情報
             *      Normal   : x,y,z法線ベクトル情報
             *      U        : テクスチャU座標
             *      V        : テクスチャV座標
             * Textures  : テクスチャファイル名（フルパス）
             * Material  : マテリアル情報
             *      Ambient  : ARGB情報
             *      Diffuse  : ARGB情報
             *      Emissive : ARGB情報
             *      Specular : ARGB情報（3DMapEditorでは不使用のため値は渡されない）
             *      
             * ※Vertexsの一次元目配列数はポリゴン構成三角形数、二次元目は点情報
             *   Vertexsの一次元目配列数とテクスチャ数は同じ
             *   VertexsとTexturesの等しいものが三角形ポリゴンに適用するテクスチャ
             *   Texturesの要素はnullの場合あり
             */
            
            XmlTextWriter writer=null;
            try{
                //ファイル名にモデル名を設定
                writer=new XmlTextWriter(path+"\\"+information.ModelName+".xml",Encoding.UTF8);
                writer.Formatting=Formatting.Indented;

                //xmlドキュメントの開始
                writer.WriteStartDocument();

                //ルートタグ開始
                writer.WriteStartElement("model");

                //モデル番号の書き込み
                writer.WriteAttributeString("no",Convert.ToString(information.ModelNo));
                
                //三角形情報の書き込み
                writer.WriteStartElement("triangles");

                for(int i=0;i<information.Vertexs.Length;i++){
                    writer.WriteStartElement("triangle");

                    writer.WriteStartElement("vertexs");
                    
                    for(int j=0;j<information.Vertexs[i].Length;j++){
                        //頂点情報の書き込み
                        writer.WriteStartElement("vertex");

                        //位置情報
                        writer.WriteElementString(
                            "position",
                            information.Vertexs[i][j].Position.X.ToString("F")+","+
                            information.Vertexs[i][j].Position.Y.ToString("F")+","+
                            information.Vertexs[i][j].Position.Z.ToString("F")
                        );

                        //法線情報
                        writer.WriteElementString(
                            "normal",
                            information.Vertexs[i][j].Normal.X.ToString("F")+","+
                            information.Vertexs[i][j].Normal.Y.ToString("F")+","+
                            information.Vertexs[i][j].Normal.Z.ToString("F")
                        );

                        //U座標
                        writer.WriteElementString(
                            "U",
                            information.Vertexs[i][j].U.ToString("F")
                        );

                        //V座標
                        writer.WriteElementString(
                            "V",
                            information.Vertexs[i][j].V.ToString("F")
                        );

                        writer.WriteEndElement();
                    }

                    writer.WriteEndElement();
                    
                    //テクスチャ情報（ファイル名）
                    if(information.Textures[i]!=null && information.Textures[i]!=""){
                        writer.WriteElementString(
                            "texture",
                            information.Textures[i].Substring(
                                information.Textures[i].LastIndexOf('\\')+1
                            )
                        );
                    }

                    writer.WriteEndElement();
                }

                writer.WriteEndElement();

                //マテリアル情報の書き込み
                writer.WriteStartElement("material");

                //Ambient
                writer.WriteElementString(
                    "ambient",
                    information.Material.Ambient.A.ToString("D")+","+
                    information.Material.Ambient.R.ToString("D")+","+
                    information.Material.Ambient.G.ToString("D")+","+
                    information.Material.Ambient.B.ToString("D")+","
                );

                //Diffuse
                writer.WriteElementString(
                    "diffuse",
                    information.Material.Diffuse.A.ToString("D")+","+
                    information.Material.Diffuse.R.ToString("D")+","+
                    information.Material.Diffuse.G.ToString("D")+","+
                    information.Material.Diffuse.B.ToString("D")+","
                );

                //Emissive
                writer.WriteElementString(
                    "emissive",
                    information.Material.Emissive.A.ToString("D")+","+
                    information.Material.Emissive.R.ToString("D")+","+
                    information.Material.Emissive.G.ToString("D")+","+
                    information.Material.Emissive.B.ToString("D")+","
                );

                writer.WriteEndElement();

                //ルートタグ終了
                writer.WriteEndDocument();
            }
            catch{
                if(writer!=null)
                    writer.Close();
            }
            writer.Close();
        }
    }
}
